unit Unit2;

interface

type
  PMSettings = ^TMPdata;
  TMSettings = packed record
    Loop: Boolean;
    Mute: Boolean;
  end;

  PTMPdata = ^TMPdata;
  TMPdata = packed record
    ClientProcess,WritePipe,ReadPipe:NativeUInt;
    FirstChance:Boolean;
    MSettings: TMSettings;
  end;


function EscapeParam(const Param:string):string;
procedure GetWidthHeight(ReadPipe: NativeUInt; var Height: Integer; var Width: Integer);
procedure Terminate(ClientProcess:NativeUInt);
function int2TMPdata(var int: NativeInt): TMPdata;
procedure SendCommand(const tmp:TMPdata; Command: AnsiString);
procedure ForceStop(tmp:TMPdata);
procedure ParseIniFile(IniFileName:string; var MP_path: string; var MP_params: string; var ViewScale: string);
function GetIniKey(const section, key, default, fnIni: string): string;
function ExpandEnvVar(const StringToExpand: string): string;

function ReadAllPipe(ReadPipe: NativeUInt; Blocked: Boolean = False): string;

implementation

uses Windows, SysUtils;
function int2TMPdata(var int: NativeInt): TMPdata;
  var pss:PTMPdata;
begin
  pss:=pointer(int);
  Result:=pss^;
end;

procedure SendCommand(const tmp:TMPdata; Command: AnsiString);
var Dummy:Cardinal;
begin
  if (tmp.ClientProcess=0) OR (tmp.WritePipe=0) then exit;
  Command := Command + #10;
  WriteFile(tmp.WritePipe, Command[1], length(Command), Dummy, nil);
end;

function EscapeParam(const Param:string):string;
begin
  if Pos(#32,Param)>0 then Result:=#34+Param+#34 else Result:=Param;
end;

procedure Terminate(ClientProcess:NativeUInt);
begin
  if ClientProcess=0 then exit;
  TerminateProcess(ClientProcess,cardinal(-1));
end;

procedure ForceStop(tmp:TMPdata);
begin
  if tmp.FirstChance then begin
    SendCommand(tmp,'quit');
    tmp.FirstChance:=false;
    if WaitForSingleObject(tmp.ClientProcess,1000)<>WAIT_TIMEOUT then exit;
  end;
  Terminate(tmp.ClientProcess);
end;

procedure GetWidthHeight(ReadPipe: NativeUInt; var Height: Integer; var Width: Integer);
  const BufSize=1024;
  var Data, conte: ansistring;
      BytesRead,aa,bb,cc:Cardinal;
      Buffer:array[0..BufSize]of ansichar;
  FI: _BY_HANDLE_FILE_INFORMATION;
begin
  Height := 0;
  Width := 0;
  Data:='';

  if not GetFileInformationByHandle(ReadPipe, FI) then
  begin
    Height := -1;
    Width := -1;
    Exit;
  end;

  repeat
    BytesRead:=0;
    if not ReadFile(ReadPipe,Buffer[0],BufSize,BytesRead,nil) then
      Break;
    Buffer[BytesRead]:=#0;
    data:=data+buffer;
  until (Pos('ID_FILENAME',Buffer)<>0) or (BytesRead = 0);

  if Pos('ID_EXIT=EOF', data) > 0 then
  begin
    Height := -1;
    Width := -1;
  end;

  repeat
    BytesRead:=0;
    if not ReadFile(ReadPipe,Buffer[0],BufSize,BytesRead,nil) then
      Break;
    Buffer[BytesRead]:=#0;
    Data:=Data+Buffer;
  until (Pos('ID_LENGTH',Buffer)<>0) or (BytesRead = 0);

  aa:=Pos('ID_VIDEO_WIDTH',data);
  if aa > 0 then
  begin
    delete(data,1,aa+14);
    conte:=copy(data,1,pos(#10,data));
    val(conte,bb,cc);
    Width:=bb;
  end;

  aa:=Pos('ID_VIDEO_HEIGHT',data);
  if aa > 0 then
  begin
    delete(data,1,aa+15);
    conte:=copy(data,1,pos(#10,data));
    val(conte,bb,cc);
    Height:=bb;
  end;

end;

function ReadAllPipe(ReadPipe: NativeUInt; Blocked: Boolean = False): string;
  const BufSize=1024;
  var Data, conte: ansistring;
      BytesRead,aa,bb,cc:Cardinal;
      Buffer:array[0..BufSize]of ansichar;
  FI: _BY_HANDLE_FILE_INFORMATION;

  Counter: Integer;
  PipeSize: DWORD;
begin
  Result := '';
  Data:='';
  Counter := 0;
  PipeSize := 0;

  if not GetFileInformationByHandle(ReadPipe, FI) then
  begin
    Exit;
  end;

  if not Blocked then
  begin
    if not PeekNamedPipe(ReadPipe, nil, 0, nil, @PipeSize, nil) then
      Exit;
    if PipeSize = 0 then
      Exit;
  end else
    PipeSize := MaxInt;

  repeat
    BytesRead:=0;
    if not ReadFile(ReadPipe,Buffer[0],BufSize,BytesRead,nil) then
      Break;
    Buffer[BytesRead]:=#0;
    data:=data+buffer;
    Counter := Counter + BytesRead;
  until (BytesRead < BufSize) or (Counter = PipeSize);

  Result := Data;
end;

procedure ParseIniFile(IniFileName:string; var MP_path: string; var MP_params: string; var ViewScale: string);
  const BufSize=1024;
  var Data: ansistring;
      BytesRead,aa:Cardinal;
      Buffer:array[0..BufSize]of ansichar;
      FileHandle: THandle;
      MP_path_x64: string;
begin
  MP_path := GetIniKey('config', 'MPlayer_path', MP_path, IniFileName);
  MP_params := GetIniKey('config', 'MPlayer_params', MP_params, IniFileName);
  ViewScale := GetIniKey('config', 'View_Size', ViewScale, IniFileName);

  MP_path := ExpandEnvVar(MP_path);

  {$IFDEF WIN64}
  MP_path_x64 := GetIniKey('config', 'MPlayer_path_x64', '', IniFileName);
  MP_path_x64 := ExpandEnvVar(MP_path_x64);
  if FileExists(MP_path_x64) then
    MP_path := MP_path_x64;
  {$ENDIF}

  EXIT;

  (*
  FileHandle:=FileOpen(IniFileName,fmOpenRead);
  Data:='';
  repeat
    BytesRead:=FileRead(FileHandle,Buffer[0],BufSize);
    Buffer[BytesRead]:=#0;
    Data:=Data+Buffer;
  until BytesRead=0;

  aa:=Pos('MPlayer_path',Data);
  Delete(Data,1,aa+13);
  MP_path:=Copy(Data,1,Pos('"',Data)-1);

  aa:=Pos('MPlayer_params',Data);
  Delete(Data,1,aa+15);
  MP_params:=Copy(Data,1,Pos('"',Data)-1);

  aa:=Pos('View_Size',Data);
  Delete(Data,1,aa+10);
  ViewScale:=Copy(Data,1,Pos('"',Data)-1);
  FileClose(FileHandle);
  *)
end;

function GetIniKey(const section, key, default, fnIni: string): string;
var
  buf: array[0..2047] of char;
begin
  GetPrivateProfileString(PChar(section), PChar(key), PChar(default),
    buf, Length(buf), PChar(fnIni));
  Result := buf;
end;

function ExpandEnvVar(const StringToExpand: string): string;
var
  Path, CurrentDir: string;
begin
  Result := '';
  if StringToExpand = '' then
    Exit;
  SetLength(Path, 3200);
  ExpandEnvironmentStrings(PChar(StringToExpand), PChar(Path), Length(Path) - 1);
  SetLength(Path, StrLen(PChar(Path)));
  CurrentDir := GetCurrentDir;
  SetCurrentDir(ExtractFileDir(GetModuleName(HInstance)));
  Result := ExpandFileName(Path);
  SetCurrentDir(CurrentDir);
end;

end.
